from .yolov5_seg import YOLOv5lSeg, YOLOv5xSeg
from .yolov8_seg import YOLOv8sSeg, YOLOv8mSeg, YOLOv8lSeg, YOLOv8xSeg
from .yolov9_seg import YOLOv9cSeg
from .yolo11_seg import YOLO11lSeg, YOLO11mSeg, YOLO11xSeg
